/* lookup_table.c -- 
 * Copyright 2004, 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 *      Rickard E. (Rik) Faith <faith@redhat.com>
 */

#include "config.h"
#include <string.h>
#include <unistd.h>

#include "libaudit.h"
#define __KERNEL__
#include <sys/syscall.h>
#undef __KERNEL__

struct transtab {
    int        value;
    const char *name;
};

struct int_transtab {
    int        key;
    unsigned int  lvalue;
};        

static struct transtab i386_syscalltab[] = {
    #include "i386_table.h"
};
#define AUDIT_I386_SYSCALL_NAMES (sizeof(i386_syscalltab)/sizeof(i386_syscalltab[0]))

static struct transtab x86_64_syscalltab[] = {
    #include "x86_64_table.h"
};
#define AUDIT_X86_64_SYSCALL_NAMES (sizeof(x86_64_syscalltab)/sizeof(x86_64_syscalltab[0]))

static struct transtab ppc64_syscalltab[] = {
    #include "ppc64_table.h"
};
#define AUDIT_PPC64_SYSCALL_NAMES (sizeof(ppc64_syscalltab)/sizeof(ppc64_syscalltab[0]))


static struct transtab ppc_syscalltab[] = {
    #include "ppc_table.h"
};
#define AUDIT_PPC_SYSCALL_NAMES (sizeof(ppc_syscalltab)/sizeof(ppc_syscalltab[0]))

static struct transtab s390x_syscalltab[] = {
    #include "s390x_table.h"
};
#define AUDIT_S390X_SYSCALL_NAMES (sizeof(s390x_syscalltab)/sizeof(s390x_syscalltab[0]))

static struct transtab s390_syscalltab[] = {
    #include "s390_table.h"
};
#define AUDIT_S390_SYSCALL_NAMES (sizeof(s390_syscalltab)/sizeof(s390_syscalltab[0]))

static struct transtab ia64_syscalltab[] = {
    #include "ia64_table.h"
};
#define AUDIT_IA64_SYSCALL_NAMES (sizeof(ia64_syscalltab)/sizeof(ia64_syscalltab[0]))

static struct transtab fieldtab[] = {
    { AUDIT_PID,      "pid"      },
    { AUDIT_UID,      "uid"      },
    { AUDIT_EUID,     "euid"     },
    { AUDIT_SUID,     "suid"     },
    { AUDIT_FSUID,    "fsuid"    },
    { AUDIT_GID,      "gid"      },
    { AUDIT_EGID,     "egid"     },
    { AUDIT_SGID,     "sgid"     },
    { AUDIT_FSGID,    "fsgid"    },
    { AUDIT_LOGINUID, "auid"     },
    { AUDIT_LOGINUID, "loginuid" },
    { AUDIT_PERS,     "pers"     },
    { AUDIT_ARCH,     "arch"     },

    { AUDIT_DEVMAJOR, "devmajor" },
    { AUDIT_DEVMINOR, "devminor" },
    { AUDIT_INODE,    "inode"    },
    { AUDIT_EXIT,     "exit"     },
    { AUDIT_SUCCESS,  "success"  },
    { AUDIT_ARG0,     "a0"       },
    { AUDIT_ARG1,     "a1"       },
    { AUDIT_ARG2,     "a2"       },
    { AUDIT_ARG3,     "a3"       },
};
#define AUDIT_FIELD_NAMES (sizeof(fieldtab)/sizeof(fieldtab[0]))

static struct transtab flagtab[] = {
    { AUDIT_FILTER_TASK,  "task"     },
    { AUDIT_FILTER_ENTRY, "entry"    },
    { AUDIT_FILTER_EXIT,  "exit"     },
    { AUDIT_FILTER_USER,  "user"     },
    { AUDIT_FILTER_WATCH, "watch"    }

};
#define AUDIT_FLAG_NAMES (sizeof(flagtab)/sizeof(flagtab[0]))

static struct transtab actiontab[] = {
    { AUDIT_NEVER,    "never"    },
    { AUDIT_POSSIBLE, "possible" },
    { AUDIT_ALWAYS,   "always"   }
};
#define AUDIT_ACTION_NAMES (sizeof(actiontab)/sizeof(actiontab[0]))

/* 
 * This table is arranged based on statistics so more frequent items
 * are at the top.
 */
static struct transtab msg_typetab[] = {
//    { AUDIT_GET,			"GET"	      			},
//    { AUDIT_SET,			"SET"	      			},
//    { AUDIT_LIST,			"LIST"	      			},
//    { AUDIT_ADD,			"ADD"	      			},
//    { AUDIT_DEL,			"DEL"	      			},
    { AUDIT_USER,			"USER"	      			},
    { AUDIT_LOGIN,			"LOGIN"       			},
//    { AUDIT_SIGNAL_INFO,		"SIGNAL_INFO" 			},
    { AUDIT_USER_AUTH,			"USER_AUTH"			},
    { AUDIT_USER_ACCT,			"USER_ACCT"			},
    { AUDIT_USER_MGMT,			"USER_MGMT"			},
    { AUDIT_CRED_ACQ,			"CRED_ACQ"			},
    { AUDIT_CRED_DISP,			"CRED_DISP"			},
    { AUDIT_USER_START,			"USER_START"			},
    { AUDIT_USER_END,			"USER_END"			},
    { AUDIT_USER_AVC,			"USER_AVC"			},
    { AUDIT_USER_CHAUTHTOK,		"USER_CHAUTHTOK"		},
    { AUDIT_USER_ERR,			"USER_ERR"			},
    { AUDIT_CRED_REFR,			"CRED_REFR"			},
    { AUDIT_USYS_CONFIG,		"USYS_CONFIG"			},
    { AUDIT_USER_LOGIN,			"USER_LOGIN"			},
    { AUDIT_USER_LOGOUT,		"USER_LOGOUT"			},
    { AUDIT_ADD_USER,			"ADD_USER"			},
    { AUDIT_DEL_USER,			"DEL_USER"			},
    { AUDIT_ADD_GROUP,			"ADD_GROUP"			},
    { AUDIT_DEL_GROUP,			"DEL_GROUP"			},
    { AUDIT_DAC_CHECK,			"DAC_CHECK"			},
    { AUDIT_CHGRP_ID,			"CHGRP_ID"			},
    { AUDIT_DAEMON_START,		"DAEMON_START"			},
    { AUDIT_DAEMON_END,			"DAEMON_END"			},
    { AUDIT_DAEMON_ABORT,		"DAEMON_ABORT"			},
    { AUDIT_DAEMON_CONFIG,		"DAEMON_CONFIG"			},
//    { AUDIT_DAEMON_RECONFIG,		"DAEMON_RECONFIG"		},
    { AUDIT_DAEMON_ROTATE,		"DAEMON_ROTATE"			},
    { AUDIT_SYSCALL,			"SYSCALL"			},
    { AUDIT_FS_WATCH,			"FS_WATCH"			},
    { AUDIT_PATH,			"PATH"				},
    { AUDIT_IPC,			"IPC"				},
    { AUDIT_SOCKETCALL,			"SOCKETCALL"			},
    { AUDIT_CONFIG_CHANGE,		"CONFIG_CHANGE" 		},
    { AUDIT_SOCKADDR,			"SOCKADDR"			},
    { AUDIT_CWD,			"CWD"				},
    { AUDIT_FS_INODE,			"FS_INODE"			},
    { AUDIT_AVC,			"AVC"				},
    { AUDIT_SELINUX_ERR,		"SELINUX_ERR"			},
    { AUDIT_AVC_PATH,			"AVC_PATH"			},
    { AUDIT_KERNEL,			"KERNEL"      			},
    { AUDIT_ANOM_LOGIN_FAILURES,	"ANOM_LOGIN_FAILURES"		},
    { AUDIT_ANOM_LOGIN_TIME,		"ANOM_LOGIN_TIME"		},
    { AUDIT_ANOM_LOGIN_SESSIONS,	"ANOM_LOGIN_SESSIONS"		},
    { AUDIT_ANOM_LOGIN_ACCT,		"ANOM_LOGIN_ACCT"		},
    { AUDIT_ANOM_LOGIN_LOCATION,	"ANOM_LOGIN_LOCATION"		},
    { AUDIT_ANOM_MAX_DAC,		"ANOM_MAX_DAC"			},
    { AUDIT_ANOM_MAX_MAC,		"ANOM_MAX_MAC"			},
    { AUDIT_ANOM_AMTU_FAIL,		"ANOM_AMTU_FAIL"		},
    { AUDIT_ANOM_RBAC_FAIL,		"ANOM_RBAC_FAIL"		},
    { AUDIT_ANOM_RBAC_INTEGRITY_FAIL,	"ANOM_RBAC_INTEGRITY_FAIL"	},
    { AUDIT_ANOM_CRYPTO_FAIL,		"ANOM_CRYPTO_FAIL"		},
    { AUDIT_ANOM_ACCESS_FS,		"ANOM_ACCESS_FS"		},
    { AUDIT_ANOM_EXEC,			"ANOM_EXEC"			},
    { AUDIT_ANOM_MK_EXEC,		"ANOM_MK_EXEC"			},
    { AUDIT_ANOM_ADD_ACCT,		"ANOM_ADD_ACCT"			},
    { AUDIT_ANOM_DEL_ACCT,		"ANOM_DEL_ACCT"			},
    { AUDIT_ANOM_MOD_ACCT,		"ANOM_MOD_ACCT"			},
    { AUDIT_RESP_ANOMALY,		"RESP_ANOMALY"			},
    { AUDIT_RESP_ALERT,			"RESP_ALERT"			},
    { AUDIT_RESP_KILL_PROC,		"RESP_KILL_PROC"		},
    { AUDIT_RESP_TERM_ACCESS,		"RESP_TERM_ACCESS"		},
    { AUDIT_RESP_ACCT_REMOTE,		"RESP_ACCT_REMOTE"		},
    { AUDIT_RESP_ACCT_LOCK_TIMED,	"RESP_ACCT_LOCK_TIMED"		},
    { AUDIT_RESP_ACCT_UNLOCK_TIMED,	"RESP_ACCT_UNLOCK_TIMED"	},
    { AUDIT_RESP_ACCT_LOCK,		"RESP_ACCT_LOCK"		},
    { AUDIT_RESP_TERM_LOCK,		"RESP_TERM_LOCK"		},
    { AUDIT_RESP_SEBOOL,		"RESP_SEBOOL"			},
    { AUDIT_RESP_EXEC,			"RESP_EXEC"			},
    { AUDIT_RESP_SINGLE,		"RESP_SINGLE"			},
    { AUDIT_RESP_HALT,			"RESP_HALT"			},
    { AUDIT_USER_ROLE_CHANGE,		"USER_ROLE_CHANGE"		},
    { AUDIT_ROLE_ASSIGN,		"ROLE_ASSIGN"			},
    { AUDIT_ROLE_REMOVE,		"ROLE_REMOVE"			},
    { AUDIT_LABEL_OVERRIDE,		"LABEL_OVERRIDE"		},
    { AUDIT_LABEL_LEVEL_CHANGE,		"LABEL_LEVEL_CHANGE"		},
    { AUDIT_USER_LABELED_EXPORT,	"USER_LABELED_EXPORT"		},
    { AUDIT_USER_UNLABELED_EXPORT,	"USER_UNLABELED_EXPORT"		}
};
#define AUDIT_MSG_TYPE_NAMES (sizeof(msg_typetab)/sizeof(msg_typetab[0]))

static struct transtab machinetab[] = {
    { MACH_X86,     "i386"   },
    { MACH_X86,     "i486"   },
    { MACH_X86,     "i586"   },
    { MACH_X86,     "i686"   },
    { MACH_86_64,   "x86_64" },
    { MACH_IA64,    "ia64"   },
    { MACH_PPC64,   "ppc64"  },
    { MACH_PPC,     "ppc"    },
    { MACH_S390X,   "s390x"  },
    { MACH_S390,    "s390"   }
};
#define AUDIT_MACHINE_NAMES (sizeof(machinetab)/sizeof(machinetab[0]))

static struct int_transtab elftab[] = {
    { MACH_X86,     AUDIT_ARCH_I386   },
    { MACH_86_64,   AUDIT_ARCH_X86_64 },
    { MACH_IA64,    AUDIT_ARCH_IA64   },
    { MACH_PPC64,   AUDIT_ARCH_PPC64  },
    { MACH_PPC,     AUDIT_ARCH_PPC    },
    { MACH_S390X,   AUDIT_ARCH_S390X  },
    { MACH_S390,    AUDIT_ARCH_S390   }
};
#define AUDIT_ELF_NAMES (sizeof(elftab)/sizeof(elftab[0]))

static int audit_lookup_name(struct transtab *table, int length,
                             const char *name)
{
	int i;
    
	for (i = 0; i < length; i++)
		if (!strcasecmp(table[i].name, name)) return table[i].value;
	return -1;
}

static const char *audit_lookup_number(struct transtab *table, int length,
                                       int number)
{
	int i;
    
	for (i = 0; i < length; i++)
		if (table[i].value == number) return table[i].name;
	return NULL;
}

int audit_name_to_field(const char *field)
{
	return audit_lookup_name(fieldtab, AUDIT_FIELD_NAMES, field);
}

const char *audit_field_to_name(int field)
{
	return audit_lookup_number(fieldtab, AUDIT_FIELD_NAMES, field);
}

int audit_name_to_syscall(const char *sc, int machine)
{
	switch (machine)
	{
		case MACH_X86:
    			return audit_lookup_name(i386_syscalltab, 
					AUDIT_I386_SYSCALL_NAMES, sc);
		case MACH_86_64:
    			return audit_lookup_name(x86_64_syscalltab, 
					AUDIT_X86_64_SYSCALL_NAMES, sc);
		case MACH_IA64:
    			return audit_lookup_name(ia64_syscalltab, 
					AUDIT_IA64_SYSCALL_NAMES, sc);
		case MACH_PPC64:
    			return audit_lookup_name(ppc64_syscalltab, 
					AUDIT_PPC64_SYSCALL_NAMES, sc);
		case MACH_PPC:
    			return audit_lookup_name(ppc_syscalltab, 
					AUDIT_PPC_SYSCALL_NAMES, sc);
		case MACH_S390X:
    			return audit_lookup_name(s390x_syscalltab, 
					AUDIT_S390X_SYSCALL_NAMES, sc);
		case MACH_S390:
    			return audit_lookup_name(s390_syscalltab, 
					AUDIT_S390_SYSCALL_NAMES, sc);
	}
	return -1;
}

const char *audit_syscall_to_name(int sc, int machine)
{
	switch (machine)
	{
		case MACH_X86:
    			return audit_lookup_number(i386_syscalltab, 
					AUDIT_I386_SYSCALL_NAMES, sc);
		case MACH_86_64:
    			return audit_lookup_number(x86_64_syscalltab, 
					AUDIT_X86_64_SYSCALL_NAMES, sc);
		case MACH_IA64:
    			return audit_lookup_number(ia64_syscalltab, 
					AUDIT_IA64_SYSCALL_NAMES, sc);
		case MACH_PPC64:
    			return audit_lookup_number(ppc64_syscalltab, 
					AUDIT_PPC64_SYSCALL_NAMES, sc);
		case MACH_PPC:
    			return audit_lookup_number(ppc_syscalltab, 
					AUDIT_PPC_SYSCALL_NAMES, sc);
		case MACH_S390X:
    			return audit_lookup_number(s390x_syscalltab, 
					AUDIT_S390X_SYSCALL_NAMES, sc);
		case MACH_S390:
    			return audit_lookup_number(s390_syscalltab, 
					AUDIT_S390_SYSCALL_NAMES, sc);
	}
	return NULL;
}

int audit_name_to_flag(const char *flag)
{
	return audit_lookup_name(flagtab, AUDIT_FLAG_NAMES, flag);
}

const char *audit_flag_to_name(int flag)
{
	return audit_lookup_number(flagtab, AUDIT_FLAG_NAMES, flag); 
}

int audit_name_to_action(const char *action)
{
	return audit_lookup_name(actiontab, AUDIT_ACTION_NAMES, action);
}

const char *audit_action_to_name(int action)
{
	return audit_lookup_number(actiontab, AUDIT_ACTION_NAMES, action);
}

// On the critical path for ausearch parser
int audit_name_to_msg_type(const char *msg_type)
{
	return audit_lookup_name(msg_typetab, AUDIT_MSG_TYPE_NAMES, msg_type);
}

const char *audit_msg_type_to_name(int msg_type)
{
	return audit_lookup_number(msg_typetab, AUDIT_MSG_TYPE_NAMES, msg_type);
}

int audit_name_to_machine(const char *machine)
{
	return audit_lookup_name(machinetab, AUDIT_MACHINE_NAMES, machine);
}

const char *audit_machine_to_name(int machine)
{
	return audit_lookup_number(machinetab, AUDIT_MACHINE_NAMES, machine);
}

unsigned int audit_machine_to_elf(int machine)
{
	unsigned int i;
    
	for (i = 0; i < AUDIT_ELF_NAMES; i++)
		if (elftab[i].key == machine) 
			return elftab[i].lvalue;
	return 0;
}

int audit_elf_to_machine(unsigned int elf)
{
	unsigned int i;
    
	for (i = 0; i < AUDIT_ELF_NAMES; i++) 
		if (elftab[i].lvalue == elf) return elftab[i].key;
	return -1;
}

